Stan Math Library: stan/math/prim/scal/prob/exp_mod_normal_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

exp_mod_normal_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_EXP_MOD_NORMAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_EXP_MOD_NORMAL_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/meta/include_summand.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <boost/random/normal_distribution.hpp>
15 #include <boost/math/special_functions/fpclassify.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18
19 namespace stan {
20
21 namespace math {
22
23 template <bool propto,
24 typename T_y, typename T_loc, typename T_scale,
25 typename T_inv_scale>
26 typename return_type<T_y, T_loc, T_scale, T_inv_scale>::type
27 exp_mod_normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma,
28 const T_inv_scale& lambda) {
29 static const char* function("stan::math::exp_mod_normal_log");
30 typedef typename stan::partials_return_type<T_y, T_loc, T_scale,
31 T_inv_scale>::type
32 T_partials_return;
33
34 using stan::is_constant_struct;
35 using stan::math::check_positive_finite;
36 using stan::math::check_finite;
37 using stan::math::check_not_nan;
38 using stan::math::check_consistent_sizes;
39 using stan::math::value_of;
40 using stan::math::include_summand;
41 using std::log;
42
43 // check if any vectors are zero length
44 if (!(stan::length(y)
45 && stan::length(mu)
46 && stan::length(sigma)
47 && stan::length(lambda)))
48 return 0.0;
49
50 // set up return value accumulator
51 T_partials_return logp(0.0);
52
53 // validate args (here done over var, which should be OK)
54 check_not_nan(function, "Random variable", y);
55 check_finite(function, "Location parameter", mu);
56 check_positive_finite(function, "Inv_scale parameter", lambda);
57 check_positive_finite(function, "Scale parameter", sigma);
58 check_consistent_sizes(function,
59 "Random variable", y,
60 "Location parameter", mu,
61 "Scale parameter", sigma,
62 "Inv_scale paramter", lambda);
63
64 // check if no variables are involved and prop-to
65 if (!include_summand<propto, T_y, T_loc, T_scale, T_inv_scale>::value)
66 return 0.0;
67
68 using boost::math::erfc;
69 using std::sqrt;
70 using std::log;
71 using std::exp;
72
73 // set up template expressions wrapping scalars into vector views
74 OperandsAndPartials<T_y, T_loc, T_scale, T_inv_scale>
75 operands_and_partials(y, mu, sigma, lambda);
76
77 VectorView<const T_y> y_vec(y);
78 VectorView<const T_loc> mu_vec(mu);
79 VectorView<const T_scale> sigma_vec(sigma);
80 VectorView<const T_inv_scale> lambda_vec(lambda);
81 size_t N = max_size(y, mu, sigma, lambda);
82
83 for (size_t n = 0; n < N; n++) {
84 // pull out values of arguments
85 const T_partials_return y_dbl = value_of(y_vec[n]);
86 const T_partials_return mu_dbl = value_of(mu_vec[n]);
87 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
88 const T_partials_return lambda_dbl = value_of(lambda_vec[n]);
89
90 const T_partials_return pi_dbl = boost::math::constants::pi<double>();
91
92 // log probability
93 if (include_summand<propto>::value)
94 logp -= log(2.0);
95 if (include_summand<propto, T_inv_scale>::value)
96 logp += log(lambda_dbl);
97 if (include_summand<propto, T_y, T_loc, T_scale, T_inv_scale>::value)
98 logp += lambda_dbl
99 * (mu_dbl + 0.5 * lambda_dbl * sigma_dbl * sigma_dbl - y_dbl)
100 + log(erfc((mu_dbl + lambda_dbl * sigma_dbl
101 * sigma_dbl - y_dbl)
102 / (sqrt(2.0) * sigma_dbl)));
103
104 // gradients
105 const T_partials_return deriv_logerfc
106 = -2.0 / sqrt(pi_dbl)
107 * exp(-(mu_dbl + lambda_dbl * sigma_dbl * sigma_dbl - y_dbl)
108 / (std::sqrt(2.0) * sigma_dbl)
109 * (mu_dbl + lambda_dbl * sigma_dbl * sigma_dbl - y_dbl)
110 / (sigma_dbl * std::sqrt(2.0)))
111 / erfc((mu_dbl + lambda_dbl * sigma_dbl * sigma_dbl
112 - y_dbl) / (sigma_dbl * std::sqrt(2.0)));
113
114 if (!is_constant_struct<T_y>::value)
115 operands_and_partials.d_x1[n]
116 += -lambda_dbl
117 + deriv_logerfc * -1.0 / (sigma_dbl * std::sqrt(2.0));
118 if (!is_constant_struct<T_loc>::value)
119 operands_and_partials.d_x2[n]
120 += lambda_dbl
121 + deriv_logerfc / (sigma_dbl * std::sqrt(2.0));
122 if (!is_constant_struct<T_scale>::value)
123 operands_and_partials.d_x3[n]
124 += sigma_dbl * lambda_dbl * lambda_dbl
125 + deriv_logerfc
126 * (-mu_dbl / (sigma_dbl * sigma_dbl * std::sqrt(2.0))
127 + lambda_dbl / std::sqrt(2.0)
128 + y_dbl / (sigma_dbl * sigma_dbl * std::sqrt(2.0)));
129 if (!is_constant_struct<T_inv_scale>::value)
130 operands_and_partials.d_x4[n]
131 += 1 / lambda_dbl + lambda_dbl * sigma_dbl * sigma_dbl
132 + mu_dbl - y_dbl + deriv_logerfc * sigma_dbl / std::sqrt(2.0);
133 }
134 return operands_and_partials.value(logp);
135 }
136
137 template <typename T_y, typename T_loc, typename T_scale,
138 typename T_inv_scale>
139 inline
140 typename return_type<T_y, T_loc, T_scale, T_inv_scale>::type
141 exp_mod_normal_log(const T_y& y, const T_loc& mu, const T_scale& sigma,
142 const T_inv_scale& lambda) {
143 return exp_mod_normal_log<false>(y, mu, sigma, lambda);
144 }
145 }
146 }
147 #endif
148
149
150
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
stan::math::exp_mod_normal_log
return_type< T_y, T_loc, T_scale, T_inv_scale >::type exp_mod_normal_log(const T_y &y, const T_loc &mu, const T_scale &sigma, const T_inv_scale &lambda)
Definition: exp_mod_normal_log.hpp:27
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::OperandsAndPartials::d_x4
VectorView< T_return_type, false, true > d_x4
Definition: OperandsAndPartials.hpp:42

[ Stan Home Page ] © 2011–2016, Stan Development Team.